export const Refuse = Symbol();
export class Accessor {
    readChild;
    writeChild;
    constructor(readChild, writeChild) {
        this.readChild = readChild;
        this.writeChild = writeChild;
    }
    and(other) {
        return new Accessor((p) => {
            const c = this.readChild(p);
            if (c === Refuse) {
                return Refuse;
            }
            return other.readChild(c);
        }, (p, gc) => {
            const c = this.readChild(p);
            if (c === Refuse) {
                return;
            }
            return other.writeChild(c, gc);
        });
    }
}
export function into(key) {
    return new Accessor((parent) => parent[key], (parent, newChild) => (parent[key] = newChild));
}
export function intoMap(key) {
    return new Accessor((parent) => parent.has(key) ? parent.get(key) : Refuse, (parent, newChild) => {
        parent.set(key, newChild);
    });
}
export function isPresent(parent) {
    return parent ?? Refuse;
}
const subscriber_queue = [];
export function readableTree(value, start = noop) {
    const { subscribe, zoom, zoomNoSet, choose, } = writableTree(value, start);
    return {
        subscribe,
        zoom,
        zoomNoSet,
        choose,
    };
}
export function writableTree(value, start = noop) {
    const tree = {
        children: new Set(),
        thisSubscribers: new Set(),
    };
    let countAllSubscribers = 0;
    let stop;
    const wt = writableTreeCore(() => value, (newValue) => (value = newValue), tree, function incrementThenStartIfNecessary() {
        ++countAllSubscribers;
        if (countAllSubscribers === 1) {
            stop = start(wt.set, wt.update) || noop;
        }
    }, function decrementThenStopIfNecessary() {
        --countAllSubscribers;
        if (countAllSubscribers === 0) {
            stop();
            stop = undefined;
        }
    }, function isReady() {
        return !!stop;
    }, []);
    return wt;
}
function writableTreeCore(getValue, writeValue, tree, incrementThenStartIfNecessary, decrementThenStopIfNecessary, isReady, parentSubscribers) {
    function set(new_value) {
        if (safe_not_equal(getValue(), new_value)) {
            writeValue(new_value);
            if (isReady()) {
                // store is ready
                const run_queue = !subscriber_queue.length;
                for (const parentSubscriber of parentSubscribers) {
                    parentSubscriber(subscriber_queue);
                }
                for (const subscriber of tree.thisSubscribers) {
                    subscriber.i();
                    const chosenValue = subscriber.g();
                    if (chosenValue !== Refuse) {
                        subscriber_queue.push(subscriber, chosenValue);
                    }
                }
                const tmpQueue = [];
                for (const getterAndChild of tree.children.values()) {
                    tmpQueue.push(getterAndChild);
                }
                let getterAndChild;
                while ((getterAndChild = tmpQueue.shift())) {
                    for (const subscribersNode of getterAndChild.c.thisSubscribers) {
                        subscribersNode.i();
                        const chosenChild = subscribersNode.g();
                        if (chosenChild !== Refuse) {
                            subscriber_queue.push(subscribersNode, chosenChild);
                        }
                    }
                    for (const childNode of getterAndChild.c.children.values()) {
                        tmpQueue.push(childNode);
                    }
                }
                if (run_queue) {
                    for (let i = 0; i < subscriber_queue.length; i += 2) {
                        subscriber_queue[i].r(subscriber_queue[i + 1]);
                    }
                    subscriber_queue.length = 0;
                }
            }
        }
    }
    function update(fn) {
        const v = getValue();
        if (v !== Refuse) {
            set(fn(v));
        }
    }
    function subscribe(run, invalidate = noop) {
        const subscriber = {
            r: run,
            i: invalidate,
            g: getValue,
        };
        tree.thisSubscribers.add(subscriber);
        incrementThenStartIfNecessary();
        const v = getValue();
        if (v !== Refuse) {
            run(v);
        }
        return () => {
            tree.thisSubscribers.delete(subscriber);
            decrementThenStopIfNecessary();
        };
    }
    function createChild(readChild, writeChild) {
        const getter = () => {
            const v = getValue();
            if (v === Refuse) {
                return Refuse;
            }
            return readChild(v);
        };
        const child = {
            g: getter,
            c: {
                children: new Set(),
                thisSubscribers: new Set(),
            },
        };
        tree.children.add(child);
        return writableTreeCore(getter, writeChild, child.c, incrementThenStartIfNecessary, decrementThenStopIfNecessary, isReady, [
            ...parentSubscribers,
            (subscriber_queue) => {
                for (const subscriber of tree.thisSubscribers) {
                    const chosenValue = subscriber.g();
                    if (chosenValue !== Refuse) {
                        subscriber.i();
                        subscriber_queue.push(subscriber, chosenValue);
                    }
                }
            },
        ]);
    }
    function zoom(accessor) {
        return createChild(accessor.readChild, (newChild) => {
            const v = getValue();
            if (v === Refuse) {
                return;
            }
            accessor.writeChild(v, newChild);
        });
    }
    function choose(readChild) {
        return createChild(readChild, writeValue);
    }
    function zoomNoSet(readChild) {
        const { subscribe, zoom: z, zoomNoSet, choose, } = zoom(new Accessor(readChild, () => {
            throw new Error("Assertion failed: writeChild for zoomNoSet must not be called!");
        }));
        return {
            subscribe,
            zoom: z,
            zoomNoSet,
            choose,
        };
    }
    return {
        set,
        update,
        subscribe,
        zoom,
        zoomNoSet,
        choose,
    };
}
// Copied from svelte/internal
function noop() {
    // Literally no operation.
}
function safe_not_equal(a, b) {
    return a != a ? b == b : a !== b || (a && typeof a === 'object') || typeof a === 'function';
}
//# sourceMappingURL=index.js.map